/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.fidentis.analyst.feature.services;

import cz.fidentis.analyst.feature.FeaturePoint;
import cz.fidentis.analyst.feature.services.FeaturePointCsvLoader;

import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.DisplayName;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotSame;

/**
 * @author Jakub Kolman
 */
public class FeaturePointExportServiceTest {

    private static final Path MODEL_TEST_FILE_DIRECTORY = Paths.get("src", "test", "resources", "cz", "fidentis", "analyst", "feature", "services");
    private static final String MODEL_FILE = "00007_01_landmarks.csv";

    @DisplayName("Test writing a CSV file")
    @Test
    void exportFeaturePointsCSVTest() throws IOException {
        // load model of feature points from file
        FeaturePointCsvExporter service = new FeaturePointCsvExporter();
        List<FeaturePoint> featurePoints = loadCsvFeaturePoints(MODEL_TEST_FILE_DIRECTORY.toString(), MODEL_FILE);
        // create tmp file to write in
        File csvOutputFile = File.createTempFile(String.format("%s_test_file_csv_landmarks", MODEL_TEST_FILE_DIRECTORY.toString()), "csv");
        service.writeToFile(featurePoints, csvOutputFile, MODEL_TEST_FILE_DIRECTORY.toString() + "/" + "test_file_csv");
        // check if model file and created file generate lists with the same elements
        List<FeaturePoint> loadedFeaturePoints = loadCsvFeaturePoints(MODEL_TEST_FILE_DIRECTORY.toString(), "test_file_csv_landmarks.csv");
        assertEquals(featurePoints.size(), loadedFeaturePoints.size());
        assertEquals(featurePoints, loadedFeaturePoints);
        assertNotSame(featurePoints, loadedFeaturePoints);
    }

    @DisplayName("Test writing a FP file")
    @Test
    void exportFeaturePointsFPTest() throws IOException {
        // load model of feature points from file
        FeaturePointFpExporter service = new FeaturePointFpExporter();
        List<FeaturePoint> featurePoints = loadCsvFeaturePoints(MODEL_TEST_FILE_DIRECTORY.toString(), MODEL_FILE);
        // create tmp file to write in
        File fpOutputFile = File.createTempFile(String.format("%s_test_file_fp_landmarks", MODEL_TEST_FILE_DIRECTORY.toString()), "fp");
        service.writeToFile(featurePoints, fpOutputFile, MODEL_TEST_FILE_DIRECTORY.toString() + "/" + "test_file_fp");
        // check if model file and created file generate lists with the same elements
        List<FeaturePoint> loadedFeaturePoints = loadFpFeaturePoints(MODEL_TEST_FILE_DIRECTORY.toString(), "test_file_fp_landmarks.fp");
        assertEquals(featurePoints.size(), loadedFeaturePoints.size());
        assertEquals(featurePoints, loadedFeaturePoints);
        assertNotSame(featurePoints, loadedFeaturePoints);
    }

    private List<FeaturePoint> loadCsvFeaturePoints(String path, String fileName) throws IOException {
        FeaturePointCsvLoader service = new FeaturePointCsvLoader();
        List<FeaturePoint> featurePoints = service.loadFeaturePoints(MODEL_TEST_FILE_DIRECTORY.toString(), MODEL_FILE);
        return featurePoints;
    }

    private List<FeaturePoint> loadFpFeaturePoints(String path, String fileName) throws IOException {
        FeaturePointCsvLoader service = new FeaturePointCsvLoader();
        List<FeaturePoint> featurePoints = service.loadFeaturePoints(MODEL_TEST_FILE_DIRECTORY.toString(), MODEL_FILE);
        return featurePoints;
    }

}
